import numpy as np
from ctypes import *

ls = 299792458. # light speed

import test

def init_rand(myseed=None):
    if(myseed is None):
        f = np.ctypeslib.load_library("qumcpy.so","./bin")
        f.init_rand_clock_.argtypes = [
            np.ctypeslib.ndpointer(dtype=np.int32)   # rnd
        ]
        f.init_rand_clock_.restype = c_void_p
        clock  = np.zeros(1,dtype=np.int32)
        f.init_rand_clock_(clock)
        chkclock = clock[0]
        return chkclock
    else:
        f = np.ctypeslib.load_library("qumcpy.so",".")
        f.init_rand_seed_.argtypes = [
            POINTER(c_int32)                          # myseed
        ]
        f.init_rand_seed_.restype = c_void_p
        seed = byref(c_int(myseed))
        f.init_rand_seed_(seed)
        return myseed

def qumc(lam2,q,u,q_err,u_err,Niter,src_delta,src_gauss,src_boxcar,par,peri):
    f = np.ctypeslib.load_library("qumcpy.so","./bin")
    f.qumc_.argtypes = [
        POINTER(c_int32),                          # Ndata
        np.ctypeslib.ndpointer(dtype=np.float64),  # lam2
        np.ctypeslib.ndpointer(dtype=np.float64),  # q
        np.ctypeslib.ndpointer(dtype=np.float64),  # u
        np.ctypeslib.ndpointer(dtype=np.float64),  # q_err
        np.ctypeslib.ndpointer(dtype=np.float64),  # u_err
        POINTER(c_int32),                          # Niter
        POINTER(c_int32),                          # src_delta
        POINTER(c_int32),                          # src_gauss
        POINTER(c_int32),                          # src_boxcar
        POINTER(c_int32),                          # Npar
        np.ctypeslib.ndpointer(dtype=np.float64),  # par
        np.ctypeslib.ndpointer(dtype=np.int32),    # peri
        np.ctypeslib.ndpointer(dtype=np.float64),  # chain
        np.ctypeslib.ndpointer(dtype=np.int32),    # Nacpt
        np.ctypeslib.ndpointer(dtype=np.int32)     # Nstep
    ]
    f.qumc_.restype = c_void_p


    Npar  = 3*src_delta + 4*src_gauss + 4*src_boxcar

    chain = np.empty([Npar+1,Niter],dtype=np.float64,order='F')
    Nacpt = np.empty([Npar],dtype=np.int32,order='F')
    Nstep = np.empty([Npar],dtype=np.int32,order='F')

    lam2  = np.array(lam2,order='F')
    q     = np.array(q,order='F')
    u     = np.array(u,order='F')
    q_err = np.array(q_err,order='F')
    u_err = np.array(u_err,order='F')

    par  = np.array(par,order='F')
    peri = np.array(peri,order='F')

    Ndata      = byref(c_int32(len(lam2)))
    Niter      = byref(c_int32(Niter))
    src_delta  = byref(c_int32(src_delta))
    src_gauss  = byref(c_int32(src_gauss))
    src_boxcar = byref(c_int32(src_boxcar))
    Npar       = byref(c_int32(Npar))

    f.qumc_(
        Ndata,lam2,q,u,q_err,u_err,
        Niter,
        src_delta,src_gauss,src_boxcar,Npar,par,peri,
        chain,Nacpt,Nstep
    )

    return chain,Nacpt,Nstep


# def qumcpy(lambda2,q,u,q_err,u_err,Niter,src_delta,src_gauss,src_boxcar,par,peri):

#     Npar  = 3*src_delta + 4*src_gauss + 4*src_boxcar

#     chain = np.empty([Npar+1,Niter],dtype=np.float64,order='F')
#     Nacpt = np.empty([Npar],dtype=np.int32,order='F')
#     Nstep = np.empty([Npar],dtype=np.int32,order='F')

#     Ndata = len(lambda2)

#     test.qumc(
#         Ndata,lambda2,q,u,q_err,u_err,
#         Niter,
#         src_delta,src_gauss,src_boxcar,Npar,par,peri,
#         chain,Nacpt,Nstep
#     )

#     return chain,Nacpt,Nstep



# # distributing beta value
# def dist_beta(Ndata,Nbeta):
#     if Nbeta < 1:
#         print('Error: "Nbeta" >= 1')
#         sys.exit()
#     elif Nbeta == 1:
#         beta = np.ones([1],dtype=np.float64)
#     else:
#         betamin = 1.0/(2*Ndata)
#         beta = np.empty([Nbeta],dtype=np.float64)
#         for i in range (Nbeta):
#             beta[i] = betamin**((Nbeta-1-i)/Nbeta)

#     return beta

