import bin.rm as rm
import numpy as np
import argparse
import bin.functions as md

import sys

# Parse the command line options
parser = argparse.ArgumentParser(description='',
                                 formatter_class=argparse.RawTextHelpFormatter)
parser.add_argument("dataFile", metavar="dataFile.dat", nargs=1,
                    help="ASCII file containing Stokes spectra & errors.")
parser.add_argument("-fmin", dest="fmin", type=float, default=700.,
                    help="float: minimum frequency [MHz]. Default is 700.")
parser.add_argument("-fmax", dest="fmax", type=float, default=1800.,
                    help="float: maximum frequency [MHz]. Default is 1800.")
parser.add_argument("-c", dest="cutoff", type=float, default=2.,
                    help="float: cutoff (threshold) of RM CLEAN. Default is 2.")
parser.add_argument("-g", dest="gain", type=float, default=0.1,
                    help="float: gain of RM CLEAN. Default is 0.1.")
parser.add_argument("-n", dest="Nmax", type=int, default=1000,
                    help="int: maximum iteration of RM CLEAN. Default is 1000")
# parser.add_argument("-p", dest="showPlots", action="store_true",
#                     help="show the plots [False].")
args = parser.parse_args()


fmin = args.fmin
fmax = args.fmax
l2min = md.FtoL2(fmax)
l2max = md.FtoL2(fmin)

print '         Frequency: %d - %d [MHz]'%(args.fmin,args.fmax)
print 'Squared wavelength: %5.3lf - %5.3lf [m^2]'%(l2min,l2max)


f = np.loadtxt(args.dataFile[0])
f = np.delete(f,np.where(f<l2min)[0][np.where(np.where(f<l2min)[1]==0)],0)
f = np.delete(f,np.where(f>l2max)[0][np.where(np.where(f>l2max)[1]==0)],0)

lam2 = f[:,0]
qdata = f[:,1]
udata = f[:,2]
qerr = f[:,3]
uerr = f[:,4]

p = rm.PolObservation(lam2,(qdata,udata),QUerr=(qerr,uerr))
phi_axis = np.arange(-250.,250.1,.1)

p.rmsynthesis(phi_axis)
p.rmclean(cutoff=args.cutoff,gain=args.gain,niter=args.Nmax)
p.plot_clean_proc()
p.plot_fdf()
